#! /usr/bin/env python3
import random
import argparse

BASES = ['A', 'C', 'G', 'T', ]

DEFAULT_GENOME_SIZE = 10 ** 6


def main(fname, genome_size):
    genome = ''.join(random.choice(BASES) for _ in range(genome_size))
    with open(fname, 'w') as f:
            f.write('>simulated\n')
            f.write(genome)
            f.write('\n')

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simulate reads form random genome with errors')
    parser.add_argument('-g', '--genome-size', type=int,
                        default=DEFAULT_GENOME_SIZE, help='Genome size')
    parser.add_argument('output', help='Output file')

    args = parser.parse_args()
    main(args.output, args.genome_size)



import itertools
import multiprocessing
from functools import lru_cache
from math import exp

import matplotlib.pyplot as plt
from covest_poisson import truncated_poisson as tr_poisson
from scipy.misc import comb

from covest import config
from covest.utils import safe_log, fix_zero


class BasicModel:
    def __init__(self, k, r, hist, max_error=None, max_cov=None, *args, **kwargs):
        self.repeats = False
        self.k = k
        self.r = r
        self.bounds = ((0.01, max_cov), (0.0, 0.5))
        self.defaults = (1, self._default_param(1))
        self.comb = [comb(k, s) * (3 ** s) for s in range(k + 1)]
        self.hist = hist
        if max_error is None:
            self.max_error = self.k + 1
        else:
            self.max_error = min(self.k + 1, max_error)

    @property
    def param_count(self):
        return len(self.bounds)

    def _default_param(self, i, default=None):
        l, r = self.bounds[i]
        if l is None or r is None:
            return default
        return (l + r) / 2

    def check_bounds(self, args):
        for arg, (l, r) in zip(args, self.bounds):
            if arg is None:
                continue
            if arg == float('NaN'):
                return False
            if (l is not None and arg < l) or (r is not None and arg > r):
                return False
        return True

    def correct_c(self, c):
        return c * (self.r - self.k + 1) / self.r

    @lru_cache(maxsize=None)
    def _get_lambda_s(self, c, err):
        return [
            c * (3 ** -s) * (1.0 - err) ** (self.k - s) * err ** s
            for s in range(self.max_error)
        ]

    def compute_probabilities(self, c, err, *_):
        # read to kmer coverage
        ck = self.correct_c(c)
        # lambda for kmers with s errors
        l_s = self._get_lambda_s(ck, err)
        # expected probability of kmers with s errors and coverage >= 1
        n_s = [self.comb[s] * (1.0 - exp(-l_s[s])) for s in range(self.max_error)]
        sum_n_s = fix_zero(sum(n_s[t] for t in range(self.max_error)))
        # portion of kmers with s errors
        a_s = [n_s[s] / sum_n_s for s in range(self.max_error)]
        # probability that unique kmer has coverage j (j > 0)
        max_hist = len(self.hist)
        p_j = [None] + [
            sum(
                a_s[s] * tr_poisson(l_s[s], j) for s in range(self.max_error)
            )
            for j in range(1, max_hist)
        ]
        return p_j

    def compute_loglikelihood(self, *args):
        if not self.check_bounds(args):
            return -config.INF
        p_j = self.compute_probabilities(*args)
        if config.ESTIMATE_TAIL:
            sp_j = sum(p_j[1:-1])
            p_j[-1] = 1 - sp_j
        return float(sum(
            self.hist[j] * safe_log(p_j[j]) for j in range(1, len(self.hist)) if self.hist[j]
        ))

    def compute_loglikelihood_multi(self, args_list, thread_count=config.DEFAULT_THREAD_COUNT):
        if thread_count is None:  # do not use multiprocessing
            likelihoods = itertools.starmap(self.compute_loglikelihood, args_list)
        else:
            pool = multiprocessing.Pool(processes=thread_count)
            likelihoods = pool.starmap(self.compute_loglikelihood, args_list)
        return {
            tuple(args): likelihood for args, likelihood in zip(args_list, likelihoods)
        }

    def plot_probs(self, est, guess, orig, cumulative=False, log_scale=True):
        def fmt(p):
            return ['{:.3f}'.format(x) if x is not None else 'None' for x in p[:20]]

        def adjust_probs(probs, hist=False):
            if not hist and config.ESTIMATE_TAIL:
                sp = sum(probs[1:-1])
                probs[-1] = 1 - sp
            if cumulative:
                return [0 if p is None else i * p for i, p in enumerate(probs)]
            else:
                return probs

        hs = float(sum(self.hist))
        hp = adjust_probs([f / hs for f in self.hist], hist=True)
        ep = adjust_probs(self.compute_probabilities(*est))
        gp = adjust_probs(self.compute_probabilities(*guess))
        if orig is not None and None not in orig:
            op = adjust_probs(self.compute_probabilities(*orig))
        else:
            op = adjust_probs([0 for _ in range(len(self.hist))])

        if log_scale:
            plt.yscale('log')
        plt.plot(
            range(len(hp)), hp, 'ko',
            label='hist',
            ms=8,
        )
        plt.plot(
            range(len(ep)), ep, 'ro',
            label='est: {}'.format(fmt(est)),
            ms=6,
        )
        plt.plot(
            range(len(gp)), gp, 'go',
            label='guess: {}'.format(fmt(guess)),
            ms=5,
        )
        plt.plot(
            range(len(op)), op, 'co',
            label='orig: {}'.format(fmt(orig)),
            ms=4,
        )
        plt.legend()
        try:
            plt.show()
        except KeyboardInterrupt:
            pass


class RepeatsModel(BasicModel):
    def __init__(self, k, r, hist, max_error=None, max_cov=None, threshold=1e-8,
                 min_single_copy_ratio=0.3, *args, **kwargs):
        super(RepeatsModel, self).__init__(k, r, hist, max_error=max_error)
        self.repeats = True
        self.bounds = (
            (0.01, max_cov), (0.0, 0.5), (min_single_copy_ratio, 0.9999), (0.0, 0.99), (0.0, 0.99))
        self.defaults = self.defaults + (self._default_param(i, default=0.5) for i in range(2, 5))
        self.threshold = threshold

    def get_hist_threshold(self, b_o, threshold):
        hist_size = len(self.hist)
        if threshold is not None:
            for o in range(1, hist_size):
                if b_o(o) <= threshold:
                    return o
        return hist_size

    @staticmethod
    def get_b_o(q1, q2, q):
        o_2 = (1 - q1) * q2
        o_n = (1 - q1) * (1 - q2) * q

        def b_o(o):
            if o == 0:
                return 0
            elif o == 1:
                return q1
            elif o == 2:
                return o_2
            else:
                return o_n * (1 - q) ** (o - 3)

        return b_o

    # noinspection PyMethodOverriding
    def compute_probabilities(self, c, err, q1, q2, q, *_):
        b_o = self.get_b_o(q1, q2, q)
        threshold_o = self.get_hist_threshold(b_o, self.threshold)
        # read to kmer coverage
        c = self.correct_c(c)
        # lambda for kmers with s errors
        l_s = self._get_lambda_s(c, err)
        # expected probability of kmers with s errors and coverage >= 1
        # noinspection PyTypeChecker
        n_os = [None] + [
            [self.comb[s] * (1.0 - exp(o * -l_s[s])) for s in range(self.max_error)]
            for o in range(1, threshold_o)
        ]
        sum_n_os = [None] + [
            fix_zero(sum(n_os[o][t] for t in range(self.max_error))) for o in range(1, threshold_o)
        ]

        # portion of kmers wit1h s errors
        # noinspection PyTypeChecker
        a_os = [None] + [
            [n_os[o][s] / (sum_n_os[o] if sum_n_os[o] != 0 else 1) for s in range(self.max_error)]
            for o in range(1, threshold_o)
        ]
        # probability that unique kmer has coverage j (j > 0)
        p_j = [None] + [
            sum(
                b_o(o) * sum(
                    a_os[o][s] * tr_poisson(o * l_s[s], j) for s in range(self.max_error)
                ) for o in range(1, threshold_o)
            ) for j in range(1, len(self.hist))
        ]
        return p_j



import time
import sys
from contextlib import contextmanager

stack = list()
messages = list()


def indent():
    if len(stack) < 2:
        return ''
    return ('| ' * (len(stack) - 2)) + '+'


def push(cnt=1):
    for _ in range(cnt):
        stack.append(time.time())


def pop(cnt=1):
    ret = None
    for _ in range(cnt):
        ret = stack.pop()
    return ret


def replace():
    pop()
    push()


def get_time(back=0):
    slen = len(stack)
    if back >= slen or back == -1:
        back = slen - 1
    return stack[slen - back - 1]


def print_all():
    global messages
    for message in messages:
        sys.stderr.write(message + "\n")
    messages = []


def msg(message, back=0):
    timediff = time.time() - get_time(back)
    messages.append(indent() + message.format(time=timediff))
    print_all()


def running_time_decorator(fn):
    def wrapped(*p, **k):
        push(2)
        ret = fn(*p, **k)
        msg("Function " + fn.__name__ + " (" + fn.__module__ +
            ") took {time} seconds.", 1)
        pop(2)
        return ret
    return wrapped


@contextmanager
def running_time(text):
    push(2)
    yield
    msg(text + " took {time} seconds.", 1)
    pop(2)



import sys
from math import log

from covest import config


def print_wrap(x, label='', cond=True):
    if cond:
        print(label, x)
    return x


def verbose_print(message):
    if not config.VERBOSE:
        return
    sys.stderr.write(message + "\n")


def safe_int(x):
    return int(x) if x != float('inf') else None


def fix_zero(x, val=1):
    if x == 0:
        return val
    else:
        return x


def safe_log(x):
    if x is None or x <= 0:
        return -config.INF
    return log(x)



#! /usr/bin/env python
import argparse
from collections import defaultdict
from covest.tools.table_generator import format_table
from covest.tools.experiment_parser import parse_all
from covest.tools import templates

SEPARATE_EF = True


def kmer_to_read_coverage(c, k, read_length=100):
    if c is not None:
        return c * read_length / (read_length - k + 1)


def compute_average(table_lines, std_key_suffix='_std'):
    table_cnt = defaultdict(lambda: defaultdict(int))
    table_sum = defaultdict(lambda: defaultdict(float))
    table_avg = defaultdict(lambda: defaultdict(float))
    table_std_sum = defaultdict(lambda: defaultdict(float))
    for key, val in table_lines.items():
        for k, v in val.items():
            try:
                table_sum[key[1:]][k] += v
                table_cnt[key[1:]][k] += 1.0
            except TypeError:
                pass

    for key, val in table_sum.items():
        for k, v in val.items():
            if table_cnt[key][k] == 0:
                table_avg[key][k] = None
            else:
                table_avg[key][k] = v / table_cnt[key][k]

    for key, val in table_lines.items():
        for k, v in val.items():
            try:
                table_std_sum[key[1:]][k] += (v - table_avg[key[1:]][k]) ** 2
            except TypeError:
                pass

    for key, val in table_std_sum.items():
        for k, v in val.items():
            if table_cnt[key][k] <= 1:
                table_avg[key][k + std_key_suffix] = 0
            else:
                table_avg[key][k + std_key_suffix] = (v / (table_cnt[key][k] - 1)) ** 0.5

    return table_avg


def main(args):
    table_lines = parse_all(args.path, args.filter, not args.no_error, legacy=args.legacy)

    header = [
        'seq_name',
        'original_coverage', 'original_error_rate', 'original_k',
        'estimated_coverage', 'estimated_error_rate',
        'estimated_genome_size',
        'estimated_q1', 'estimated_q2', 'estimated_q',
        'guessed_coverage', 'guessed_error_rate',
        'original_loglikelihood', 'estimated_loglikelihood', 'guessed_loglikelihood',
    ]

    header_avg = [
        'original_coverage',
        'original_error_rate',
        'original_k',
        'estimated_coverage', 'estimated_coverage_std',
        'estimated_error_rate', 'estimated_error_rate_std',
        'estimated_genome_size', 'estimated_genome_size_std',
        'estimated_q1', 'estimated_q1_std',
        'estimated_q2', 'estimated_q2_std',
        'estimated_q', 'estimated_q_std',
        'guessed_coverage', 'guessed_coverage_std',
        'guessed_error_rate', 'guessed_error_rate_std',
        'original_loglikelihood', 'original_loglikelihood_std',
        'estimated_loglikelihood', 'estimated_loglikelihood_std',
        'guessed_loglikelihood', 'guessed_loglikelihood_std',
    ]

    # header = [
    #     'original_coverage', 'original_error_rate',
    #     'estimated_coverage', 'estimated_error_rate',
    # ]

    format_templates = {
        'html': templates.html,
        'csv': templates.csv,
        'tex': templates.tex,
    }

    format_escape = {
        'tex': lambda x: x.replace('_', '\\_'),
    }

    titles = {
        'original_coverage': 'Coverage',
        'original_error_rate': 'Error Rate',
        'estimated_coverage': 'Est. Coverage',
        'estimated_coverage_std': 'Est. Coverage Std',
        'estimated_error_rate': 'Est. Error Rate',
        'estimated_error_rate_std': 'Est. Error Rate Std',
        'estimated_genome_size': 'Est. Genome Size',
        'estimated_genome_size_std': 'Est. Genome Size Std',
    }

    if args.average:
        table_lines = compute_average(table_lines)
        header = header_avg

    print(format_table(
        header,
        titles,
        sorted(
            list(table_lines.values()),
            key=lambda x: (
                x['original_coverage'],
                x['original_error_rate'],
                x['original_k'],
                x.get('repeats', False),
                x['seq_name'],
            )
        ),
        template_file=format_templates[args.format],
        escape=format_escape.get(args.format, None),
    ))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Parse experiment output and generate table')
    parser.add_argument('path', help='Experiment')
    parser.add_argument('-f', '--format', default='html', help='Table format')
    parser.add_argument('-i', '--filter', default='*.out', help='Filter files')
    parser.add_argument('-a', '--average', action='store_true',
                        help='Compute average from all sequences')
    parser.add_argument('-ne', '--no-error', action='store_true', help='Error is unknown')
    parser.add_argument('--legacy', action='store_true', help='Run in legacy mode')
    args = parser.parse_args()
    main(args)



#! /usr/bin/env python3
import random
import argparse
from os import path
from Bio import SeqIO


BASES = ['A', 'C', 'G', 'T', ]

DEFAULT_COVERAGE = 10
DEFAULT_ERROR_RATE = .03
DEFAULT_READ_LENGTH = 100


def other(base):
    b = random.randrange(len(BASES) - 1)
    if BASES[b] == base:
        return BASES[-1]
    return BASES[b]


def reverse_complement(seq):
    complement = {
        'G': 'C',
        'C': 'G',
        'A': 'T',
        'T': 'A',
        'N': 'N',
    }
    return ''.join(complement[x] for x in reversed(seq))


def load_reads(fname):
    _, ext = path.splitext(fname)
    fmt = 'fasta'
    if ext == '.fq' or ext == '.fastq':
        fmt = 'fastq'
    with open(fname, "rU") as f:
        for read in SeqIO.parse(f, fmt):
            yield read.seq


def main(fname, genome_file, read_length, error_rate, coverage, error_free_fname=None):
    genome = list(load_reads(genome_file))[0].upper()
    genome_size = len(genome)
    read_count = int(round((coverage * genome_size) / float(read_length)))
    with open(fname, 'w') as f:
        if error_free_fname:
            eff = open(error_free_fname, 'w')

        for i in range(read_count):
            pos = random.randrange(genome_size - read_length)
            original_read = genome[pos:pos + read_length]
            original_read = original_read if random.randrange(2)\
                else reverse_complement(original_read)
            read = ''.join(b if random.random() >= error_rate else other(b)
                           for b in original_read)
            f.write('>read_{}-{}\n'.format(i, pos))
            f.write('{}\n'.format(read))
            if error_free_fname:
                eff.write('>read_{}-{}\n'.format(i, pos))
                eff.write('{}\n'.format(original_read))

        if error_free_fname:
            eff.close()


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simulate reads form random genome with errors')
    parser.add_argument('-c', '--coverage', type=float,
                        default=DEFAULT_COVERAGE, help='Coverage')
    parser.add_argument('-e', '--error-rate', type=float,
                        default=DEFAULT_ERROR_RATE, help='Error rate')
    parser.add_argument('-r', '--read-length', type=int,
                        default=DEFAULT_READ_LENGTH, help='Read length')
    parser.add_argument('-f', '--error-free', help='Error free output file')
    parser.add_argument('genome', help='Genome file')
    parser.add_argument('output', help='Output file')

    args = parser.parse_args()

    main(args.output, args.genome, args.read_length, args.error_rate,
         args.coverage, args.error_free)



#!/usr/bin/env python3.5
import argparse
import json
import os
import shutil
import subprocess
import sys

from pathlib import Path


def main(args):
    # create dir
    p = Path(args.name)
    if not args.force and p.exists():
        print('Path already exists. Use --force to use it.', file=sys.stderr)
        exit(1)
    p.mkdir(exist_ok=True)

    # gather data
    if args.datasource:
        ds = Path(args.datasource)
        if ds.exists() and ds.is_dir():
            for f in ds.iterdir():
                shutil.copy2(str(f), str(p))
        elif ds.exists() and not os.access(str(ds), os.X_OK):
            shutil.copy2(str(ds), str(p))
        else:
            print('Executing "%s":' % ds, file=sys.stderr)
            ret = subprocess.call([str(ds.resolve())], shell=True)
            if ret:
                print('"%s" failed with return code:%d' % (ds, ret), file=sys.stderr)
                exit(2)
            else:
                print('"%s" succeeded.' % (ds,), file=sys.stderr)

    # copy runner script
    if args.runner:
        r = Path(args.runner)
        if r.exists():
            shutil.copy2(str(r), str(p))
        else:
            print('File does not exist: %s' % r, file=sys.stderr)
            exit(2)

    if args.config:
        cfg = p / 'config.json'
        cfg_json = json.loads(args.config)
        json.dumpf(cfg_json, cfg.open(mode='w'))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Prepare experiment')
    parser.add_argument('name', help='directory')
    parser.add_argument('-t', '--type', type=str, choices=['simulated', 'real'],
                        default='simulated', help='Type of experiment')
    parser.add_argument('-d', '--datasource', type=str,
                        help='Data source: either generation script or directory'
                             ' or kmer histogram file')
    parser.add_argument('-r', '--runner', type=str,
                        help='Experiment runner: script for running the experiment')
    parser.add_argument('-c', '--config', type=str,
                        help='parameters for the experiment runner')
    parser.add_argument('-f', '--force', action='store_true',
                        help='force use existing path')

    main(parser.parse_args())



MAX_EXP = 200
GRID_DEPTH = 3
INF = float('inf')
VERBOSE = True
ESTIMATE_TAIL = True
PLOT_LOG_SCALE = True
USE_BIGFLOAT = False
STEP = 1.1
OPTIMIZATION_METHOD = 'L-BFGS-B'
# OPTIMIZATION_METHOD  = 'TNC'
INITIAL_GRID_COUNT = 20
INITIAL_GRID_STEP = 3
DEFAULT_SAMPLE_FACTOR = 2
DEFAULT_SAMPLE_TARGET = 50
DEFAULT_ERR_SCALE = 10
DEFAULT_K = 21
DEFAULT_READ_LENGTH = 100
DEFAULT_REPEAT_MODEL = 0
DEFAULT_MIN_SINGLECOPY_RATIO = 0.3
try:
    from multiprocessing import cpu_count

    DEFAULT_THREAD_COUNT = cpu_count()
except NotImplementedError:
    DEFAULT_THREAD_COUNT = 2



import json
import math
import random
from collections import defaultdict, namedtuple
from functools import lru_cache
from os import path

from scipy.stats import binom

from covest_poisson import poisson

from . import config
from .utils import safe_int, verbose_print


def get_trim(hist, precision=0):
    ss = float(sum(hist))
    s = 0.0
    trim = None
    for i, h in enumerate(hist):
        s += h
        r = s / ss
        if precision:
            r = round(r, precision)
        if r == 1 and trim is None:
            trim = i
    return trim


def sample_hist(hist, factor=2, trim=None):
    if trim is None:
        if len(hist) > 200:
            trim = get_trim(hist, 5)
        else:
            trim = len(hist)
    else:
        trim = min(len(hist), trim * factor)

    h = [0 for _ in range(trim)]
    prob = 1.0 / factor
    max_h = 0
    for i, v in enumerate(hist):
        if i >= trim:
            break
        if i < 100:
            b = binom(i, prob)
            probs = [b.pmf(j) for j in range(1, i + 1)]
        else:
            probs = [poisson(i * prob, j) for j in range(1, i + 1)]

        for j, p in enumerate(probs):
            h[j + 1] += v * p

    for i, v in enumerate(h):
        d = v - round(v)
        if random.random() < d:
            h[i] = math.ceil(v)
        else:
            h[i] = math.floor(v)
        if h[i]:
            max_h = i
    return h[:max_h + 1]


def auto_sample_hist(hist, target_size=50, sample_factor=2):
    h = list(hist)
    f = 1
    while len(h) > target_size:
        h = sample_hist(h, sample_factor)
        f *= sample_factor
    return h, f


def load_hist(fname, tail_sum=False, auto_trim=None, trim=None, auto_sample=None,
              sample_factor=None):
    hist = defaultdict(int)
    max_hist = 0

    with open(fname, 'r') as f:
        for line in f:
            l = line.split()
            i = int(l[0])
            cnt = int(l[1])
            hist[i] = cnt
            max_hist = max(max_hist, i)

    hist_l = [hist[b] for b in range(max_hist + 1)]
    if auto_sample is None and sample_factor is not None:
        hist_l = sample_hist(hist_l, sample_factor, trim)
    if auto_sample is not None:
        if sample_factor is None:
            sample_factor = config.DEFAULT_SAMPLE_FACTOR
        hist_l, sample_factor = auto_sample_hist(hist_l, auto_sample, sample_factor)
        verbose_print('Histogram sampled with factor {}.'.format(sample_factor))
    hist_trimmed = list(hist_l)
    if auto_trim is not None:
        trim = get_trim(hist_l, auto_trim)
        verbose_print('Trimming at: {}'.format(trim))
        hist_trimmed = hist_l[:trim]
    elif trim is not None:
        hist_trimmed = hist_l[:trim]
    if tail_sum:
        tail = sum(hist_l[trim:]) if trim is not None else 0
        hist_trimmed.append(tail)
    return hist_l, hist_trimmed, sample_factor


@lru_cache(maxsize=None)
def count_reads_size(fname):
    from Bio import SeqIO
    _, ext = path.splitext(fname)
    fmt = 'fasta'
    if ext == '.fq' or ext == '.fastq':
        fmt = 'fastq'
    try:
        with open(fname, "rU") as f:
            return sum(len(read) for read in SeqIO.parse(f, fmt))
    except FileNotFoundError as e:
        verbose_print(e)


def parse_data(data):
    model_class_name = data['model']

    guess = list()
    guess.append(data.get('guessed_coverage', None))
    guess.append(data.get('guessed_error_rate', None))
    if model_class_name == 'RepeatsModel':
        guess.append(data.get('guessed_q1', None))
        guess.append(data.get('guessed_q2', None))
        guess.append(data.get('guessed_q', None))

    estimated = list()
    estimated.append(data.get('estimated_coverage', None))
    estimated.append(data.get('estimated_error_rate', None))
    if model_class_name == 'RepeatsModel':
        estimated.append(data.get('estimated_q1', None))
        estimated.append(data.get('estimated_q2', None))
        estimated.append(data.get('estimated_q', None))

    return namedtuple('ParsedData', ('estimated', 'guess'))(estimated=estimated, guess=guess)


def print_output(
    model,
    estimated=None,
    guess=None,
    orig_coverage=None,
    orig_error_rate=None,
    orig_q1=None,
    orig_q2=None,
    orig_q=None,
    sample_factor=None,
    repeats=False,
    silent=False,
    reads_size=None,
):
    output_data = dict()
    output_data['model'] = model.__class__.__name__

    if guess is not None:
        output_data['guessed_loglikelihood'] = model.compute_loglikelihood(*guess)
        output_data['guessed_coverage'] = guess[0] * sample_factor
        output_data['guessed_error_rate'] = guess[1]
        if repeats:
            output_data['guessed_q1'] = guess[2]
            output_data['guessed_q2'] = guess[3]
            output_data['guessed_q'] = guess[4]

    if estimated is not None:
        output_data['estimated_loglikelihood'] = model.compute_loglikelihood(*estimated)
        output_data['estimated_coverage'] = estimated[0] * sample_factor
        output_data['estimated_error_rate'] = estimated[1]
        if repeats:
            output_data['estimated_q1'] = estimated[2]
            output_data['estimated_q2'] = estimated[3]
            output_data['estimated_q'] = estimated[4]
            if orig_q1 is None:
                orig_q1 = estimated[2]
            if orig_q2 is None:
                orig_q2 = estimated[3]
            if orig_q is None:
                orig_q = estimated[4]

        output_data['estimated_genome_size'] = safe_int(round(
            sum(
                i * h for i, h in enumerate(model.hist)
            ) / model.correct_c(estimated[0])
        ))

        if reads_size is not None:
            output_data['estimated_genome_size_r'] = safe_int(
                round(reads_size / (estimated[0] * sample_factor))
            )

    if orig_error_rate is not None:
        output_data['original_error_rate'] = orig_error_rate
    elif estimated is not None:
        orig_error_rate = estimated[1]

    if orig_coverage is not None:
        if repeats:
            output_data['original_loglikelihood'] = model.compute_loglikelihood(
                orig_coverage, orig_error_rate, orig_q1, orig_q2, orig_q
            )
        else:
            output_data['original_loglikelihood'] = model.compute_loglikelihood(
                orig_coverage, orig_error_rate
            )

    output_data['hist_size'] = len(model.hist)
    output_data['tail_included'] = config.ESTIMATE_TAIL
    output_data['sample_factor'] = sample_factor

    if not silent:
        print(json.dumps(
            output_data, sort_keys=True, indent=4, separators=(',', ': ')
        ))

    return output_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools.core import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

try:
    with open('README.rst') as readme_file:
        readme = readme_file.read()
except FileNotFoundError:
    readme = ''

requirements = [
    'biopython',
    'matplotlib',
    'numpy',
    'pystache',
    'scipy<0.16',
    'pyfasta',
]

test_requirements = [
]

covest_poisson = Extension(
    'covest_poisson',
    libraries=['m'],
    sources=['c_src/covest_poissonmodule.c'],
    extra_compile_args=['-std=c99'],
)

setup(
    name='covest',
    version='0.1.0',
    description="Covest estimates the coverage and genom size, "
    "just from k-mer abundance histogram computed from DNA sequences reads.",
    long_description=readme,
    author="Michal Hozza",
    author_email='mhozza@gmail.com',
    url='https://github.com/mhozza/covest',
    packages=[
        'covest',
    ],
    package_dir={
        'covest': 'covest'
    },
    include_package_data=True,
    package_data={
        'covest': ['templates/*.tpl'],
    },
    ext_modules=[covest_poisson],
    install_requires=requirements,
    entry_points={
        'console_scripts': ['covest=covest.covest:run'],
    },
    scripts=[
        'bin/gsest.py',
        'bin/generate_sequence.py',
        'bin/read_simulator.py',
        'bin/prepare_experiment.py',
        'bin/experiment_table.py',
    ],
    license='GNU GPLv3',
    zip_safe=False,
    keywords='covest',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



#! /usr/bin/env python3
import argparse
from covest.covest import count_reads_size


def main(args):
    rc = count_reads_size(args.reads)
    if args.coverage:
        print('Genome size:', int(round(rc / args.coverage)))
    if args.genome_size:
        print('Coverage:', rc / args.genome_size)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simulate reads form random genome with errors')
    parser.add_argument('reads', help='Input histogram')
    parser.add_argument('-c', '--coverage', type=float, help='Coverage')
    parser.add_argument('-g', '--genome-size', type=float, help='Genome size')

    args = parser.parse_args()
    main(args)



import argparse
import json
from math import exp
from multiprocessing import Pool

from scipy.optimize import minimize

from . import config
from .grid import initial_grid, optimize_grid
from .inverse import inverse
from .data import count_reads_size, load_hist, parse_data, print_output
from .models import BasicModel, RepeatsModel
from .perf import running_time, running_time_decorator
from .utils import verbose_print


def estimate_p(cc, alpha):
    return (cc * (alpha - 1)) / (alpha * cc - alpha - cc)


def compute_coverage_apx(hist, k, r):
    def fix_coverage(coverage):
        return inverse(lambda c: (c - c * exp(-c)) / (1 - exp(-c) - c * exp(-c)))(coverage)

    def kmer_to_read_coverage(coverage):
        return coverage * r / (r - k + 1)

    observed_ones = hist[1]
    all_kmers = sum(i * h for i, h in enumerate(hist))
    total_unique_kmers = sum(h for h in hist)

    if total_unique_kmers == 0:
        return 0.0, 1.0

    # discard first column
    all_kmers -= observed_ones
    unique_kmers = total_unique_kmers - observed_ones
    # compute coverage from hist >=2
    try:
        cov = all_kmers / unique_kmers
        cov = fix_coverage(cov)
        # fix unique kmers
        unique_kmers /= (1.0 - exp(-cov) - cov * exp(-cov))
        # compute alpha (error read ratio)
        estimated_ones = unique_kmers * cov * exp(-cov)
        estimated_zeros = unique_kmers * exp(-cov)
        error_ones = max(0.0, observed_ones - estimated_ones)
        alpha = error_ones / (total_unique_kmers + estimated_zeros)
        # estimate probability of correct kmer and error rate
        estimated_p = max(0.0, estimate_p(cov, alpha))
        e = 1 - estimated_p ** (1.0 / k)
        # return corrected coverage and error estimate
        if estimated_p > 0:
            # function for conversion between kmer and base coverage
            return float(kmer_to_read_coverage(cov / estimated_p)), float(e)
        else:
            return 0.0, float(e)
    except ZeroDivisionError:
        return 0.0, 1.0


class CoverageEstimator:
    GRID_SEARCH_TYPE_NONE = 0
    GRID_SEARCH_TYPE_PRE = 1
    GRID_SEARCH_TYPE_POST = 2

    def __init__(self, model, err_scale=1, fix=None):
        self.model = model
        self.fix = fix
        self.err_scale = err_scale
        self.bounds = list(self.model.bounds)
        self.bounds[1] = self.bounds[1][0], self.bounds[1][1] * self.err_scale

    def likelihood_f(self, x):
        args = list(x)
        if self.fix is not None:
            args = [j if self.fix[i] is None else self.fix[i] for i, j in enumerate(args)]
        args[1] /= self.err_scale
        return -self.model.compute_loglikelihood(*args)

    def _optimize(self, r):
        return minimize(
            self.likelihood_f, r,
            method=config.OPTIMIZATION_METHOD,
            bounds=self.bounds,
            options={'disp': False}
        )

    def compute_coverage(
            self,
            guess,
            grid_search_type=GRID_SEARCH_TYPE_PRE,
            n_threads=config.DEFAULT_THREAD_COUNT,
    ):
        r = list(guess)
        r[1] *= self.err_scale
        success = True
        try:
            verbose_print('Bounds: {}'.format(self.bounds))
            if grid_search_type == self.GRID_SEARCH_TYPE_NONE:
                with running_time('First optimization'):
                    res = self._optimize(r)
                    success = res.success
                    if not success:
                        verbose_print(
                            'Optimization unsuccessful.\n'
                            'Initial params:{}\nResult{}'.format(r, res)
                        )
                    r = res.x
            else:
                params = initial_grid(r, bounds=self.bounds)
                with running_time('Initial grid optimization'):
                    min_r = None
                    with Pool(n_threads) as pool:
                        results = list(pool.map(self._optimize, params))
                    for res in results:
                        if min_r is None or min_r > res.fun:
                            min_r = res.fun
                            success = res.success
                            if not success:
                                verbose_print(
                                    'Optimization unsuccessful.\n'
                                    'Initial params:{}\nResult{}'.format(r, res)
                                )
                            r = res.x

            if grid_search_type == self.GRID_SEARCH_TYPE_POST and not success:
                verbose_print('Starting grid search with guess: {}'.format(r))
                r = optimize_grid(
                    self.likelihood_f, r, bounds=self.bounds,
                    fix=self.fix, n_threads=n_threads,
                )
        except KeyboardInterrupt:
            pass
        verbose_print('Status: %s' % ('success' if success else 'failure',))
        r[1] /= self.err_scale
        return r


@running_time_decorator
def main(args):
    # Load histogram
    hist_orig, hist, sample_factor = load_hist(
        args.input_histogram, tail_sum=config.ESTIMATE_TAIL, auto_trim=args.auto_trim,
        trim=args.trim, auto_sample=args.auto_sample, sample_factor=args.sample_factor,
    )
    reads_size = None
    if args.read_file is not None:
        reads_size = count_reads_size(args.read_file)
    err_scale = args.error_scale
    if sample_factor and args.coverage:
        args.coverage /= sample_factor

    # Model selection
    if args.repeats:
        model_class = RepeatsModel
    else:
        model_class = BasicModel
    # Model initialisation
    model = model_class(
        args.kmer_size, args.read_length, hist,
        max_error=8, max_cov=args.max_coverage,
        min_single_copy_ratio=args.min_q1,
    )

    orig = (args.coverage, args.error_rate, args.q1, args.q2, args.q)[:model.param_count]
    fix = orig if args.fix else None

    if args.ll_only:
        ll = model.compute_loglikelihood(*orig)
        print('Loglikelihood:', ll)
    else:
        if args.load:  # Load saved data, don't estimate anything
            with open(args.load) as f:
                parsed_data = parse_data(json.load(f))
                guess = parsed_data.guess
                res = parsed_data.estimated
        else:
            verbose_print('Estimating coverage for {}'.format(args.input_histogram))
            # Compute initial guess
            if args.start_original:
                guess = list(orig)
            else:
                guess = list(model.defaults)
                cov, e = compute_coverage_apx(hist_orig, args.kmer_size, args.read_length)
                if not (cov == 0 and e == 1):  # We were able to guess cov and e
                    guess[:2] = cov, e
                if fix:
                    for i, v in fix:
                        if v is not None:
                            guess[i] = v
            verbose_print('Initial guess: {} ll:{}'.format(
                guess, model.compute_loglikelihood(*guess)
            ))

            # Estimate coverage
            estimator = CoverageEstimator(model, err_scale=err_scale, fix=fix)
            res = estimator.compute_coverage(
                guess,
                grid_search_type=args.grid,
                n_threads=args.thread_count,
            )

            print_output(
                model,
                res, guess, args.coverage, args.error_rate, args.q1, args.q2, args.q,
                sample_factor=sample_factor, repeats=args.repeats, reads_size=reads_size,
            )

        # Draw plot
        if args.plot is not None:
            model.plot_probs(
                res, guess, orig, cumulative=args.plot, log_scale=config.PLOT_LOG_SCALE
            )


def run():
    parser = argparse.ArgumentParser(description='Simulate reads form random genome with errors')
    parser.add_argument('input_histogram', type=str, help='Input histogram')
    parser.add_argument('--load', type=str, help='Load json')
    parser.add_argument('-k', '--kmer-size', type=int,
                        default=config.DEFAULT_K, help='Kmer size')
    parser.add_argument('-r', '--read-length', type=int,
                        default=config.DEFAULT_READ_LENGTH, help='Read length')
    parser.add_argument('--plot', type=bool, nargs='?', const=False,
                        help='Plot probabilities (use --plot 1 to plot probs * j)')
    parser.add_argument('-rp', '--repeats', action='store_true', help='Estimate with repeats')
    parser.add_argument('-ll', '--ll-only', action='store_true',
                        help='Only compute log likelihood')
    parser.add_argument('-M', '--max-coverage', type=int, help='Upper coverage limit')
    parser.add_argument('-t', '--trim', type=int, help='Trim histogram at this value')
    parser.add_argument('-at', '--auto-trim', type=int, nargs='?', const=0,
                        help='Trim histogram automatically with this threshold')
    parser.add_argument('-sf', '--sample-factor', type=int,
                        help='Sample histogram with this factor')
    parser.add_argument('-as', '--auto-sample', type=int, nargs='?',
                        const=config.DEFAULT_SAMPLE_TARGET,
                        help='Sample histogram automatically to this target size')
    parser.add_argument('-g', '--grid', type=int, default=0,
                        help='Grid search type: 0 - None, 1 - Pre-grid, 2 - Post-grid')
    parser.add_argument('-e', '--error-rate', type=float, help='Error rate')
    parser.add_argument('-es', '--error-scale', type=float, default=config.DEFAULT_ERR_SCALE,
                        help='Error scale')
    parser.add_argument('-c', '--coverage', type=float, help='Coverage')
    parser.add_argument('-q1', type=float, help='q1')
    parser.add_argument('-mq1', '--min-q1', type=float, default=config.DEFAULT_MIN_SINGLECOPY_RATIO,
                        help='minimum single copy ratio')
    parser.add_argument('-q2', type=float, help='q2')
    parser.add_argument('-q', type=float, help='q')
    parser.add_argument('-so', '--start-original', action='store_true',
                        help='Start form given values')
    parser.add_argument('-f', '--fix', action='store_true',
                        help='Fix some vars, optimize others')
    parser.add_argument('-T', '--thread-count', default=config.DEFAULT_THREAD_COUNT, type=int,
                        help='Thread count')
    parser.add_argument('-s', '--genome-size', dest='read_file',
                        help='Calculate genome size from reads')

    main(parser.parse_args())


if __name__ == '__main__':
    run()



# Binary Search
def inverse_bs(f, precision=1e-8):
    """Given a function y = f(x) that is a monotonically increasing function on
    non-negative numbers, return the function x = f_1(y) that is an approximate
    inverse, picking the closest value to the inverse, within delta."""
    def f_1(y):
        low, high = 0, float(y)
        last, mid = 0, high / 2
        while abs(mid - last) > precision:
            if f(mid) < y:
                low = mid
            else:
                high = mid
            last, mid = mid, (low + high) / 2
        return mid
    return f_1


# Newton's Method
def inverse(f, delta=1e-8):
    """Given a function y = f(x) that is a monotonically increasing function on
    non-negative numbers, return the function x = f_1(y) that is an approximate
    inverse, picking the closest value to the inverse, within delta."""
    def derivative(func):
        return lambda y: (func(y + delta) - func(y)) / delta

    def root(y):
        return lambda x: f(x) - y

    def newton(y, iters=15):
        guess = float(y) / 2
        rootfunc = root(y)
        derifunc = derivative(rootfunc)
        d = (rootfunc(guess) / derifunc(guess))
        while abs(d) > delta:
            guess -= d
            d = (rootfunc(guess) / derifunc(guess))

        return guess
    return newton



import itertools
import pickle
import random
from multiprocessing import Pool

from . import config
from .perf import running_time, running_time_decorator
from .utils import verbose_print


def unpack_call(args):
    f, data = args
    f = pickle.loads(f)
    return f(data)


@running_time_decorator
def optimize_grid(fn, initial_guess, bounds=None, maximize=False, fix=None,
                  n_threads=config.DEFAULT_THREAD_COUNT):
    def generate_grid(args, step, max_depth):
        def generate_grid_single(var, fix=None):
            if fix is None:
                return (
                    var * step ** d
                    for d in range(-max_depth, max_depth + 1) if d != 0
                )
            else:
                return [fix]

        def filter_bounds(var_grid, i):
            if bounds is None or len(bounds) <= i or len(bounds[i]) != 2:
                return var_grid
            low, high = bounds[i]
            return (
                var for var in var_grid
                if (low is None or var >= low) and (high is None or var <= high)
            )

        var_grids = [
            list(filter_bounds(generate_grid_single(var, fix[i]), i))
            for i, var in enumerate(args)
        ]
        return itertools.product(*var_grids)

    if fix is None:
        fix = [None] * len(initial_guess)
    sgn = -1 if maximize else 1
    f = pickle.dumps(fn, pickle.HIGHEST_PROTOCOL)
    min_val = sgn * unpack_call([f, initial_guess])
    min_args = initial_guess
    step = config.STEP
    grid_depth = config.GRID_DEPTH
    diff = 1
    n_iter = 0
    try:
        while diff > 0.1 or step > 1.001:
            n_iter += 1
            diff = 0.0
            grid = list(generate_grid(min_args, step, grid_depth))
            verbose_print('Iter : {}, Grid size: {}'.format(n_iter, len(grid)))
            fn_grid = zip([f] * len(grid), grid)
            with running_time('grid iteration'):
                with Pool(n_threads) as pool:
                    res = pool.map(unpack_call, fn_grid)
            for args, val in zip(grid, res):
                # val = fn(args)
                if sgn * val < min_val:
                    diff += min_val - val
                    min_val = sgn * val
                    min_args = args
            if diff < 1.0:
                step = 1 + (step - 1) * 0.75
            verbose_print('d:{} s:{}'.format(diff, step))
            verbose_print('New args: {}, ll: {}'.format(min_args, min_val))
    except KeyboardInterrupt:
        verbose_print('Grid search interrupted')

    verbose_print('Number of iterations in grid search:{}'.format(n_iter))
    return min_args


def initial_grid(initial_guess, count=config.INITIAL_GRID_COUNT, bounds=None, fix=None, ):
    def generate_grid(step):
        def apply_bounds(interval, i):
            if bounds is None or len(bounds) <= i or len(bounds[i]) != 2:
                return interval
            lb, rb = bounds[i]
            li, ri = interval
            if lb is not None:
                li = max(li, lb)
            if rb is not None:
                ri = min(ri, rb)
            return li, ri

        def generate_random_params():
            bounds = [
                apply_bounds((var / step, var * step), i) for i, var in enumerate(initial_guess)
            ]
            return [
                random.uniform(*interval) if fix is None or fix[i] is None else fix[i]
                for i, interval in enumerate(bounds)
            ]

        if count < 1:
            grid = []
        else:
            grid = [initial_guess]
            for _ in range(count - 1):
                grid.append(generate_random_params())
        return grid

    if fix is None:
        fix = [None] * len(initial_guess)
    return list(generate_grid(config.INITIAL_GRID_STEP))
